﻿using System.Collections;
using System.Collections.Generic;
using UnityEngine;

public class AudioManager : MonoBehaviour
{
    public static AudioManager Instance;
    public AudioSource _backgroundBGM;
    public AudioClip _dayBGM;
    public AudioClip _nightBGM;
    public AudioSource _actionAudioSource;
    // public AudioClip[]AudioClipList;
    public AudioClip RudderAudio;
    public AudioClip EatAudio;
    public AudioClip RepairAudio;
    public AudioClip RefillAudio; // light up
    public AudioClip CannonAudio;
    public AudioClip CrashAudio; // colliding

    public enum AudioType
    {
        RudderAudio,
        EatAudio,
        RefillAudio,
        CannonAudio,
        CrashAudio,
    }

    public void PlayBGM(UIManger.GameWeatherState gameState)
    {
        _backgroundBGM.loop = true;
        _backgroundBGM.volume = 0.2f;
        // type day or night
        Debug.Log("gameState: " + gameState);
        switch (gameState)
        {
            case UIManger.GameWeatherState.day:
                Debug.Log("_dayBGM");
                _backgroundBGM.clip = _dayBGM;
                _backgroundBGM.Play();
                return;
            case UIManger.GameWeatherState.night:
                Debug.Log("_nightBGM");
                _backgroundBGM.clip = _nightBGM;
                _backgroundBGM.Play();
                return;
            default:
                return;
        }
    }

    public void PlayAudio(AudioType type)
    {
        switch (type)
        {
            case AudioType.CannonAudio:
                _actionAudioSource.PlayOneShot(CannonAudio, 1);
                return;
            case AudioType.RudderAudio:
                _actionAudioSource.clip = RudderAudio;
                _actionAudioSource.loop = true;
                _actionAudioSource.Play();
                return;
            case AudioType.EatAudio:
                _actionAudioSource.PlayOneShot(EatAudio, 1);
                return;
            case AudioType.CrashAudio:
                _actionAudioSource.PlayOneShot(CrashAudio, 1);
                return;
            case AudioType.RefillAudio:
                _actionAudioSource.PlayOneShot(RefillAudio, 1);
                return;
            default:
                return;
        }
    }

    public void StopAudio()
    {
        _actionAudioSource.loop = false;
        _actionAudioSource.Stop();
    }

    public void stopBGM()
    {
        _backgroundBGM.Stop();
    }

    void Awake()
    {
        if (Instance != null)
        {
            Destroy(gameObject);
        }
        Instance = this;
    }
    void Start()
    {
    }

    // Update is called once per frame
    void Update()
    {

    }
}
