﻿using UnityEngine;
using System.Collections;

public class Obstacle : MonoBehaviour
{
    public enum Type
    {
        still, motional
    }

    public enum Direction
    {
        left, right, top, bottom
    }

    public Type type = Type.still;
    public Direction direction = Direction.left;
    public int hp = 1;
    public float dyingTime = 1f;
    public bool canMove = false;

    private Vector2 currentPos;
    private Vector2 startingPos;

    public float movingSpeed = 0.02f;
    public float shakeSpeed = 50f; // how fast it shake
    public float shakeDelta = 0.08f; // how much it shake

    public bool canDamage
    {
        get
        {
            return (this.hp > 0);
        }
    }

    private void Start()
    {
        this.currentPos = transform.position;
        this.startingPos = this.currentPos;
    }

    private void Update()
    {
        if (canMove)
        {
            float finalSpeed = (direction == Direction.left || direction == Direction.bottom) ? -movingSpeed : movingSpeed;
            Vector3 v3 = new Vector3(0, 0, 0);
            if (direction == Direction.left || direction == Direction.right)
            {
                v3 = new Vector3(finalSpeed, 0, 0);
            }
            else
            {
                v3 = new Vector3(0, finalSpeed, 0);
            }
            this.gameObject.transform.position += v3;
        }
        if (!canDamage)
        {
            Vector3 v = transform.position;
            v.y += shakeDelta * Mathf.Sin(Time.time * shakeSpeed);
            transform.position = v;
        }
    }

    private void OnBecameVisible()
    {
        if (type == Type.motional)
        {
            this.canMove = true;
        }
    }

    public void collide()
    {
        this.hp--;
        if (!canDamage)
        {
            this.animateDestroy();
        }
    }

    private void animateDestroy()
    {
        if (type == Type.motional)
        {
            Destroy(this.gameObject);
        }
        else
        {
            Destroy(this.gameObject, this.dyingTime);
        }
        
    }
}
