using System.Collections;
using System.Collections.Generic;
using UnityEngine;
using UnityEngine.InputSystem;
using UnityEngine.UI;
using UnityEngine.Tilemaps;
using System.Linq;

public class Player : MonoBehaviour
{
    //[SerializeField] float movementSpeed = 3f;

    Vector2 _Movement;
    public Rigidbody2D _Rigidbody;
    Collider2D _Collider;
    GameManager gm;

    // public int hp = 6;
    public int hunger = 6;
    CraftingTable onTouchedCraftingTable;
    CraftingTable onControllingCraftingTable;
    RaftBlock onStandingBlock;

    public Image[] hearts;
    public Sprite halfHeartLeft;
    public Sprite FullHeart;
    public Sprite halfHeartRight;
    public GameObject ActionProgressBar;
    public Slider _slider;
    public Text _progressText;

    Vector3 m_Size;
    // public int hunger = 100;
    //private int _MaxHP = 100;
    public double _hp;
    public bool isAlive = true;
    public float movementSpeed = 3f;
    public float swimmingSpeed = 0.5f;
    public bool isOnRaft = true;
    public Animator _animator;

    public double HP
    {
        get { return _hp; }
        set
        {
            if (_hp <= 0)
            {
                _hp = 0;
                // Die()
                isAlive = false;
            }
            else if (_hp >= GameManager.Instance._MaxHP)
            {
                _hp = GameManager.Instance._MaxHP;
            }
            _hp = value;
        }
    }



    // public Player(int DefaultHP)
    // {
    //     this.HP = DefaultHP;
    // }
    void CheckHearts()
    {
        if (HP >= 0)
        {
            for (int i = 0; i < hearts.Length; i++)
            {
                if (i < HP)
                {
                    hearts[i].sprite = FullHeart;
                    hearts[i].enabled = true;
                }
                else
                {
                    hearts[i].enabled = false;
                }
            }
            if ((HP % 2) != 0 && !(HP >= hearts.Length))
                hearts[(int)(HP)].sprite = halfHeartLeft;
        }
    }

    void UpdateMovementSpeed(double percentage)
    {
        float CountedSpeed = GameManager.Instance._PlayerSpeed * (float)percentage;
        if (CountedSpeed >= 0.5f)
        {
            movementSpeed = CountedSpeed;
        }
        else
        {
            movementSpeed = 0.5f;
        }

    }

    public void SetProgressActive()
    {
        // set active
        ActionProgressBar.SetActive(true);
    }

    public void StartProgress(float currentProgress, float maxProgress)
    {
        // update progress bar on time
        SetProgressActive();
        float progress = 1 - (currentProgress / maxProgress / 0.9f);
        _slider.value = progress;
        // Debug.Log("currentProgress / maxProgress : " + progress);
        _progressText.text = Mathf.FloorToInt(progress * 100f).ToString();
    }

    public void FinishProgress()
    {
        // set InActive , Reset default
        _slider.value = 0f;
        ActionProgressBar.SetActive(false);
    }

    void DropHp()
    {
        _hp -= 0.5;
    }

    private void Awake()
    {
        Debug.Log("Awake()");
        _Rigidbody = GetComponent<Rigidbody2D>();
        _Collider = GetComponent<Collider2D>();
        gm = GameObject.Find("GameManager").GetComponent<GameManager>();

    }

    private void Start()
    {
        ActionProgressBar.SetActive(false);
        _Collider = GetComponent<Collider2D>();
        m_Size = _Collider.bounds.size;
        InvokeRepeating("DropHp", 2.0f, (float)GameManager.Instance._MindlessHpInterval);
    }

    private void Update()
    {
        if (isAlive)
        {
            float percentage = ((float)HP - 0) / ((float)GameManager.Instance._MaxHP - 0);
            UpdateMovementSpeed(percentage);
        }


        CheckHearts();
        this.checkStandingPosition();

        // force _slider on top of player 
        _slider.transform.position = Camera.main.WorldToScreenPoint(new Vector3(this.transform.position.x, this.transform.position.y + m_Size.y + 1.5f));

        // add charter animation
        if (_animator)
        {
            if (!isAlive)
                _animator.SetBool("IsDead", true);

            if (this.isOnRaft)
                _animator.SetBool("isSwiming", false);
            else
                _animator.SetBool("isSwiming", true);

            if ((_Movement.x + _Movement.y) != 0)
                _animator.SetBool("IsMoving", true);
            else
                _animator.SetBool("IsMoving", false);
        }

        if (this.onControllingCraftingTable == null)
        { // not touching anything
            this.move();
        }
        else
        {

            if (this.onControllingCraftingTable.tag == "FoodBox")
            {
                //this.onControllingCraftingTable.controlMovement(this._Movement);
                //_Rigidbody.velocity = _Movement * movementSpeed;
            }
            else if (
               this.onControllingCraftingTable.tag == "Rudder" ||
               this.onControllingCraftingTable.tag == "Harpoon")
            {
                this.onControllingCraftingTable.controlMovement(this._Movement);
            }
            else if (this.onStandingBlock.tag == "RaftBlock")
            {

            }
        }
    }

    private void move()
    {
        float finalSpeed = movementSpeed;
        if (!this.isOnRaft)
        {
            finalSpeed *= swimmingSpeed;
        }
        Vector3 finalVelocity = _Movement * finalSpeed;
        if (!this.isOnRaft)
        {
            finalVelocity -= new Vector3(gm.waterResistantSpeed, 0, 0);
        }
        _Rigidbody.velocity = finalVelocity;
    }

    public bool GetPlayerStatus()
    {
        // Debug.Log($"{name} player HP : " + HP);
        // Debug.Log($"{name} player isAlive : " + isAlive);
        return isAlive;
    }

    public void OnMovement(InputAction.CallbackContext context)
    {
        if (!this.isAlive)
        {
            return;
        }
        _Movement = context.ReadValue<Vector2>();
        if (_animator)
        {
            _animator.SetFloat("Horizontal", _Movement.x);
            _animator.SetFloat("Vertical", _Movement.y);
        }
    }

    public void OnInteraction(InputAction.CallbackContext context)
    {
        if (!this.isAlive)
        {
            return;
        }
        var action = context.phase;
        switch (context.phase)
        {
            case InputActionPhase.Started:
                Debug.Log("Started");
                if (this.onTouchedCraftingTable != null)
                {
                    this.onControllingCraftingTable = this.onTouchedCraftingTable;
                    this.onControllingCraftingTable.controllingPlayer = this;
                    this.onControllingCraftingTable.controlAction(action);
                }
                else if (this.onStandingBlock != null)
                {
                    this.onControllingCraftingTable = this.onStandingBlock;
                    this.onControllingCraftingTable.controllingPlayer = this;
                    this.onStandingBlock.controlAction(action);

                }
                break;
            case InputActionPhase.Performed:
                Debug.Log("Performed");
                if (this.onControllingCraftingTable != null)
                {
                    this.onControllingCraftingTable.controlAction(action);
                }
                break;
            case InputActionPhase.Canceled:
                Debug.Log("Canceled");
                if (this.onControllingCraftingTable != null)
                {
                    this.onControllingCraftingTable.controlAction(action);
                    this.onControllingCraftingTable = null;
                }
                this.freeze(false);


                break;
        }

    }

    public void checkStandingPosition()
    {

        RaycastHit2D[] allHits = Physics2D.RaycastAll(transform.position, transform.TransformDirection(Vector2.zero));
        // filter all raft and raft block objects
        RaycastHit2D[] hits = allHits.Where(hit => hit.collider.gameObject.tag == "Raft" || hit.collider.gameObject.tag == "RaftBlock").ToArray();
        // check is on the raft
        this.isOnRaft = hits.Length != 0;
        // check standing on which raft block
        foreach (RaycastHit2D hit in hits)
        {
            if (hit && hit.collider.gameObject.tag == "RaftBlock")
            {
                //Debug.Log(this.name + " is hitting " + hit.collider.name + " of " + hit.collider.tag);
                this.onStandingBlock = hit.collider.GetComponent<RaftBlock>();
                break;
            }
        }

        this.transform.parent = (this.isOnRaft) ? gm.raft.transform : gm.world.transform;

    }

    void OnBecameInvisible()
    {
        this.HP = 0;
        this.isAlive = false;
        // this.freeze(true);
    }
    private void FixedUpdate()
    {

    }

    private void OnCollisionEnter2D(Collision2D collision)
    {
        //Debug.Log("OnCollisionEnter2D: is touching " + collision.gameObject.tag + " of " + collision.gameObject.name);
        if (this.onTouchedCraftingTable != null)
        {
            this.onTouchedCraftingTable.highlight(false); // unhighlight previous touching objects
        }
        if (collision.gameObject.tag == "Rudder" && collision.gameObject.GetComponent<Rudder>() != null)
        {
            this.onTouchedCraftingTable = collision.gameObject.GetComponent<Rudder>();
            this.onTouchedCraftingTable.highlight(true);
        }
        else if (collision.gameObject.tag == "FoodBox" && collision.gameObject.GetComponent<FoodBox>() != null)
        {
            this.onTouchedCraftingTable = collision.gameObject.GetComponent<FoodBox>();
            this.onTouchedCraftingTable.highlight(true);
        }
        else if (collision.gameObject.tag == "WoodBox" && collision.gameObject.GetComponent<WoodBox>() != null)
        {
            this.onTouchedCraftingTable = collision.gameObject.GetComponent<WoodBox>();
            this.onTouchedCraftingTable.highlight(true);
        }
        else if (collision.gameObject.tag == "Harpoon" && collision.gameObject.GetComponent<Harpoon>() != null)
        {
            this.onTouchedCraftingTable = collision.gameObject.GetComponent<Harpoon>();
            this.onTouchedCraftingTable.highlight(true);
        }
        else if (collision.gameObject.tag == "Lantern" && collision.gameObject.GetComponent<Lantern>() != null)
        {
            this.onTouchedCraftingTable = collision.gameObject.GetComponent<Lantern>();
            this.onTouchedCraftingTable.highlight(true);
        }
        else if (collision.gameObject.tag == "Obstacle" && collision.gameObject.GetComponent<Harpoon>() != null)
        {
            Debug.Log("collide with obstacle");
        }
        else if (collision.gameObject.tag == "Suppliment" && collision.gameObject.GetComponent<Harpoon>() != null)
        {

        }
        else if (collision.gameObject.tag == "Player")
        {
            Debug.Log(this.name + "is colliding with " + collision.gameObject.name);
        }
    }

    private void OnTriggerEnter2D(Collider2D collision)
    {
        //Debug.Log("OnTriggerEnter2D: is trigger" + collision.gameObject.name);
    }

    private void OnTriggerExit2D(Collider2D collision)
    {
        //Debug.Log("OnTriggerExit2D: is trigger" + collision.gameObject.name);
    }

    private void OnCollisionExit2D(Collision2D collision)
    {
        if (this.onTouchedCraftingTable == null) { return; }
        if (
            this.onTouchedCraftingTable.tag == "Rudder" ||
            this.onTouchedCraftingTable.tag == "FoodBox" ||
            this.onTouchedCraftingTable.tag == "Harpoon" ||
            this.onTouchedCraftingTable.tag == "RaftBlock"
            ) // touching rudder
        {
            this.onTouchedCraftingTable.highlight(false);
            this.onTouchedCraftingTable.controllingPlayer = null;
            this.onTouchedCraftingTable = null;
        }

    }

    public void freeze(bool isTrigger)
    {
        if (isTrigger)
        {
            this._Rigidbody.constraints = RigidbodyConstraints2D.FreezeAll;
        }
        else
        {
            this._Rigidbody.constraints = RigidbodyConstraints2D.None;
            this._Rigidbody.constraints = RigidbodyConstraints2D.FreezeRotation;

        }
    }
}

