﻿using System.Collections;
using System.Collections.Generic;
using UnityEngine;
using UnityEngine.Tilemaps;
using UnityEngine.InputSystem;

public class RaftBlock : CraftingTable
{
    private static int max_hp = 4;
    public int hp = 3;
    private GameManager gm;
    private SpriteRenderer spriteRenderer;
    public Raft raft;
    public Sprite normalWoodSprite;
    public Sprite damagedWoodSprite;
    public Sprite heavyDamagedWoodSprite;
    public Sprite armedWoodSprite;
    private bool untouchable = false;

    public int currentWoodNum
    {
        get { return 1; }
        set { }
    }

    public static float maxRepairingSeconds = 5;
    public float repairingTimeRemaining = 5; // 5s
    public bool isRepairing = false;

    private Vector3Int previous;

    private void Awake()
    {
        this.gm = GameObject.Find("GameManager").GetComponent<GameManager>();
        this.spriteRenderer = this.GetComponent<SpriteRenderer>();
    }

    void Start()
    {
        this.updateRaft();
    }

    override public void controlMovement(Vector2 _Movement)
    {
        //this.CancelRepairing();
    }

    override public void controlAction(InputActionPhase action)
    {
        base.controlAction(action);
        switch (action)
        {
            case InputActionPhase.Started:
                this.StartRepairing();
                AudioManager.Instance.PlayAudio(AudioManager.AudioType.RefillAudio);
                break;
            case InputActionPhase.Performed:
                this.OnRepairing();
                break;
            case InputActionPhase.Canceled:
                // this.controllingPlayer = null;
                AudioManager.Instance.StopAudio();
                this.CancelRepairing();
                break;
        }
    }

    public void StartRepairing()
    {
        Debug.Log("try repairing...");
        if (this.hp >= max_hp)
        {
            Debug.Log("already armed");
            return;
        }
        if (gm._woodNum2 > 0)
        {
            Debug.Log("start repairing");
            this.isRepairing = true;
        }
        else
        {
            Debug.Log("not enough wood");
        }
    }

    public void OnRepairing()
    {

    }

    public void FinishRepairing()
    {
        Debug.Log("finish repairing");
        gm._woodNum2--;
        this.repairRaftBlock();
        this.ResetTable();
    }

    public void CancelRepairing()
    {
        Debug.Log("cancel repairing");
        this.ResetTable();
    }

    public void ResetTable()
    {
        if (!this.isRepairing) { return; }
        this.isRepairing = false;
        this.repairingTimeRemaining = maxRepairingSeconds;
        this.freeze(false);
        // Animation end
        Debug.Log("this.controllingPlayer" + this.controllingPlayer);
        this.controllingPlayer.FinishProgress();
        this.controllingPlayer = null;
    }

    public void damageRaftBlock()
    {
        this.hp--;
        this.updateRaft();
        StartCoroutine(changeToUntouchable());
    }

    public void repairRaftBlock()
    {
        if (this.hp >= max_hp)
        {
            Debug.Log(this.spriteRenderer.gameObject.name + "is at max health");
            this.updateRaft();
            return;
        }
        this.hp++;
        this.updateRaft();
    }

    private void updateRaft()
    {
        Debug.Log("update raft UI at " + this.spriteRenderer.gameObject.name);
        switch (this.hp)
        {
            case 0:
                Destroy(this.gameObject, 1);
                break;
            case 1:
                this.spriteRenderer.sprite = this.heavyDamagedWoodSprite;
                break;
            case 2:
                this.spriteRenderer.sprite = this.damagedWoodSprite;
                break;
            case 3:
                this.spriteRenderer.sprite = this.normalWoodSprite;
                break;
            case 4:
                this.spriteRenderer.sprite = this.armedWoodSprite;
                break;
        }
    }

    private void Update()
    {
        if (this.isRepairing)
        {

            if (this.repairingTimeRemaining > 0)
            {
                this.controllingPlayer.StartProgress(this.repairingTimeRemaining -= Time.deltaTime, maxRepairingSeconds);
                this.repairingTimeRemaining -= Time.deltaTime;
            }
            else
            {
                this.FinishRepairing();
            }
        }
    }

    private void OnTriggerEnter2D(Collider2D collision)
    {
        if (this.untouchable) { return; }
        if (collision.GetComponent<Obstacle>() != null)
        {
            Debug.Log(this.name + " is hiting obstacle");
            Obstacle obstacle = collision.gameObject.GetComponent<Obstacle>();
            AudioManager.Instance.PlayAudio(AudioManager.AudioType.CrashAudio);
            this.damageRaftBlock();
            obstacle.collide();

        }
    }

    private IEnumerator changeToUntouchable()
    {
        this.untouchable = true;
        yield return new WaitForSeconds(1f);
        // do something here.
        this.untouchable = false;

    }
}
