﻿using System.Collections;
using System.Collections.Generic;
using UnityEngine;
using TMPro;
using UnityEngine.InputSystem;
using UnityEngine.EventSystems;
using UnityEngine.UI;
using System.Linq;
using UnityEngine.Experimental.Rendering.Universal;

public class UIManger : MonoBehaviour
{
    private AudioManager am;
    public GameObject UI_Canvas;
    public GameObject GameOverUI;
    public GameObject WinGameUI;
    public GameObject StartMenuUI;
    public GameObject HUD;
    public GameObject Ship;
    public GameObject[] Pointer;
    GraphicRaycaster ui_raycaster;
    public TMP_Text GameProgressBar;
    PointerEventData click_data;
    List<RaycastResult> click_results;
    public int Curr = 0;
    public int Next = 1;
    private Vector3 startPos;
    private Vector3 endPos;
    private Vector3 defaultPos;
    public GameWeatherState CurrentState = GameWeatherState.day;
    public GameWeatherState NextState = GameWeatherState.night;
    public TMP_Text GameScore;
    public float CurrentLevelTime = 10;
    public float NextLevelTime = 5;
    private float counter;
    public Light2D EnvLight;
    public Light2D[] RodLights;
    public Light2D[] playerLight;
    public bool IsSwitching = false;
    private float last_p = 0;
    public float progress = 0;
    float repeat = 0;
    public float SwitchDayDuration = 3f;
    private bool nextDay = false;
    public GameObject _dayBar;
    public GameObject _nightBar;
    public enum GameWeatherState
    {
        day,
        night
    }

    private void Awake()
    {
    }

    private void Start()
    {

        ui_raycaster = UI_Canvas.GetComponent<GraphicRaycaster>();
        click_data = new PointerEventData(EventSystem.current);
        click_results = new List<RaycastResult>();

        defaultPos = Pointer[0].transform.position;
        Ship.transform.position = defaultPos;
        startPos = Pointer[0].transform.position;
        endPos = Pointer[1].transform.position;
    }

    void Update()
    {
        if (CurrentState == GameWeatherState.day)
        {
            ChangeToDay();
            _dayBar.SetActive(true);
            _nightBar.SetActive(false);
        }
        else
        {
            ChangeToNight();
            _dayBar.SetActive(false);
            _nightBar.SetActive(true);
        }

        MoveShipFish(CurrentLevelTime);
        // EnvLight.intensity = Mathf.Lerp(EnvLight.intensity, 0.15f, SwitchDayDuration * Time.deltaTime);
        // MoveToNextWave(nextDay, 5);
        // checkSwitchNextLevel(IsSwitching);
        int s = (int)GameManager.Instance.GameScore;
        GameScore.text = s.ToString();

        if (Mouse.current.leftButton.wasReleasedThisFrame)
        {
            GetUIElementClicked();
        }
    }

    void GetUIElementClicked()
    {
        Debug.Log("GetUIElementClicked");
        click_data.position = Mouse.current.position.ReadValue();
        click_results.Clear();

        ui_raycaster.Raycast(click_data, click_results);

        foreach (RaycastResult result in click_results)
        {
            GameObject ui_element = result.gameObject;
            Debug.Log("raycasting " + ui_element.name);
            if (ui_element.tag != "Untagged")
            {
                Debug.Log(ui_element.tag);
                CheckAction(ui_element.tag);
            }
        }
    }

    void CheckAction(string tag)
    {
        switch (tag)
        {
            case "Restart":
                GameManager.Instance.RestartGame();
                break;
            case "Start":
                GameManager.Instance.StartGame();
                break;
            default:
                return;
        }
    }
    public void MoveShipFish(float time)
    {
        repeat = Mathf.Repeat(Time.time, time);
        progress = Mathf.InverseLerp(0, time, repeat);

        Ship.transform.position = Vector3.Lerp(startPos, endPos, progress);
        // Debug.Log(" progress: " + progress + " repeat : " + repeat);
        // if ((progress - last_p) < 0)
        CurrentState = NextState;
        last_p = progress;
    }
    public void MoveToNextWave(bool nextIsDay, int nextLevelTime)
    {
        // float distance = Vector3.Distance(startPos, endPos);
        float distanceCheck = Vector3.Distance(Ship.transform.position, endPos);
        CurrentLevelTime = nextLevelTime;
        if (!nextIsDay)
        {
            NextState = GameWeatherState.night;
        }
        else
        {
            NextState = GameWeatherState.day;
        }
        AudioManager.Instance.PlayBGM(NextState);
        // IsSwitching = false;
        // nextDay = !nextDay;
        // float Speed = distance / time;
        // Debug.Log(distanceCheck);
        // float lerpValue = Mathf.Lerp(startPos.x, endPos.x, 0.1f);
        // Debug.Log("lerpValue : " + lerpValue);
        // Ship.transform.position = new Vector3(Ship.transform.position.x + lerpValue, 0, 0);
        // float a = Mathf.Lerp(startPos, endPos, 0.2);
        // Ship.transform.position = Vector3.MoveTowards(Ship.transform.position, endPos, Time.deltaTime * Speed);

        // repeat = Mathf.Repeat(Time.time, nextLevelTime);
        // progress = Mathf.InverseLerp(0, nextLevelTime, repeat);
        // Ship.transform.position = Vector3.Lerp(startPos, endPos, progress);

        // Debug.Log(" progress: " + progress + " repeat : " + repeat);


    }

    void checkSwitchNextLevel(bool isNextLevel)
    {
        // run get next level
        if (isNextLevel)
        {
            // switch mode
            if (CurrentState == GameWeatherState.day)
            {
                // night mode 
                ChangeToNight();
                CurrentState = GameWeatherState.night;
            }
            else
            {
                // day mode
                ChangeToDay();
                CurrentState = GameWeatherState.day;
            }
            CurrentLevelTime = NextLevelTime;
            // SetNextPosition();
            IsSwitching = false;
        }
    }


    void ChangeToDay()
    {
        EnvLight.intensity = Mathf.Lerp(EnvLight.intensity, 1.1f, SwitchDayDuration * Time.deltaTime);
        for (int i = 0; i < playerLight.Length; i++)
        {
            playerLight[i].intensity = 0;
        }

        for (int k = 0; k < RodLights.Length; k++)
        {
            RodLights[k].pointLightOuterRadius = 0;
        }
    }

    void ChangeToNight()
    {
        EnvLight.intensity = Mathf.Lerp(EnvLight.intensity, 0.15f, SwitchDayDuration * Time.deltaTime);
        for (int i = 0; i < playerLight.Length; i++)
        {
            playerLight[i].intensity = 1;
        }

        for (int k = 0; k < RodLights.Length; k++)
        {
            RodLights[k].pointLightOuterRadius = 5f;
        }
    }

    public void SetNextPosition()
    {
        Vector3 s = startPos;
        startPos = endPos;
        endPos = startPos;
        // Curr = Next;
        // if ((Next + 1) >= Pointer.Length)
        // {
        //     //    Next = 0; reset position
        //     Ship.transform.position = defaultPos;
        //     Next = 1;
        //     Curr = 0;
        // }
        // else
        // {
        //     Next += 1;
        // }
        // startPos = Pointer[Curr].transform.position;
        // endPos = Pointer[Next].transform.position;
    }
}
